<?php
/**
 * @file
 *  Drupal State module
 */

/**
 * Implements drupal_get_form().
 *
 * Defines configuration form fields.
 *
 * @param array $form
 * @param array &$form_state
 * @return array
 *   - the return value of system_settings_form()
 */
function _state_admin_form($form, &$form_state) {
  drupal_add_css(
      ($path = drupal_get_path('module', 'state')) . '/admin/state.admin.css',
      array('type' => 'file', 'group' => CSS_DEFAULT, 'preprocess' => FALSE, 'every_page' => FALSE)
  );
  drupal_add_js(
      $path . '/admin/state.admin.js',
      array('type' => 'file', 'group' => JS_DEFAULT, 'preprocess' => FALSE, 'every_page' => FALSE)
  );

  $timeout_cookie = (int) ini_get('session.cookie_lifetime');
  $timeout_server = (int) ini_get('session.gc_maxlifetime');
  $timeout_warning = '';
  if ($timeout_cookie) {
    $timeout_warning .= '<p class="warning">' . t('Session prolongation isn\'t applicable when the session cookie doesn\'t expire when the browser is closed, check session.cookie_lifetime in settings.php (should be zero).') . '</p>';
  }
  if ($timeout_server > state::FORM_EXPIRE) {
    $timeout_warning .= '<p class="warning">'
        . t(
            'Current session timeout (life on server) is !timeout minutes. A lower value, like 1800 (a half an hour) is recommended.!breakA Drupal form expires after !hrs hours, thus a higher value than that makes little sense. Check session.gc_maxlifetime in settings.php.',
            array('!break' => '<br/>', '!timeout' => round($timeout_server / 60, 1), '!hrs' => state::FORM_EXPIRE / (60 * 60))
        ) . '</p>';
  }

  //  Documentation.
  $doc = '<h5>'
      . t(
          'More detailed documentation available here: <a href="@doc_backend" target="_blank">PHP</a> and <a href="@doc_frontend" target="_blank">Javascript</a>.',
          array(
              '@doc_backend' => 'http://www.simple-complex.net/docs/drupal/modules/state/php/state_8module.html',
              '@doc_frontend' => 'http://www.simple-complex.net/docs/drupal/modules/state/js/symbols/State.html'
          )
      )
      . '</h5>'
      . '<div class="columns-2"><div><h5>' . t('PHP functions') . '</h5>';
  if (function_exists('inspect_doc_functions')
      && ($funcs = inspect_doc_functions('state', '', array('state_preprocess_html', 'state_ajax_callback')))
  ) {
    foreach ($funcs as $v) {
      $doc .= '<p>' . ($v[2] ? ($v[2] . '<br/>') : '')
          . '<span class="code emph">' . $v[0] . '</span>'
          . '<span class="code">' . $v[1] . '</span></p>';
    }
    $doc .= '</div><div><h5>' . t('Javascript methods') . '</h5>'
        . '<p>' . t('The library is available as Drupal.State and State.') . '</p>';
    $funcs = array(
      array(
          'State.alive',
          '()',
          'Check whether session seems alive and current, and form hasnt expired.',
      ),
      array(
          'State.pageResponseNumber',
          '()',
          'Page load number; how many times backend State has positively detected that a (any) page was delivered.',
      ),
      array(
          'State.sessionProlonged',
          '(date)',
          'Tell State that the current session got prolonged now, due to a (non jQuery) AJAX or iframe request.',
      ),
      array(
          'State.inspect',
          '()',
          'Inspect current state.',
      ),
      array(
          'State.testCheck',
          '()',
          'Do immediate check.',
      ),
      array(
          'State.testInfo',
          '()',
          'Test info (dialog) method.',
      ),
    );
    foreach ($funcs as $v) {
      $doc .= '<p>' . ($v[2] ? ($v[2] . '<br/>') : '')
            . '<span class="code emph">' . $v[0] . '</span>'
            . '<span class="code">' . $v[1] . '</span></p>';
    }
    $doc .= '</div></div>';
  }
  $form['functions'] = array(
      '#type' => 'fieldset',
      '#title' => t('PHP and Javascript'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      'content' => array(
        '#type' => 'markup',
        '#markup' => $doc,
      ),
  );

  //  Help.
  $form['help'] = array(
      '#type' => 'fieldset',
      '#title' => t('Help'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      'content' => array(
        '#type' => 'markup',
        '#markup' =>
'<h5>' . t('Unconditional versus user-activity conditioned session prolongation') . '</h5>
<p>' . t('Unconditional: the visitors\' session will be kept alive no matter if the visitor is active or not - only limited by form expiration (!hrs hours).',
    array('!hrs' => state::FORM_EXPIRE / (60 * 60))
) . '</p>
<p>' . t('Conditional: the visitors session will be prolonged for as long as the visitor seems active - e.g. recently has moved the mouse; pressed a key; scrolled the window.'
) . '</p>
<h5>' . t('Getting close to session/form expiration') . '</h5>
<p>' . t('10 to 20% before session/form expiration frontend State pops a dialog box telling the user that the page is about to expire - \'please save your work...\'.'
) . '</p>
<h5>' . t('Session or form has expired, or session isn\'t current') . '</h5>
<p>' . t('If a window/tab\'s session or form has expired - or the visitor has a new session or no session at all:
!break- State will pop a dialog box informing the visitor about the state of the affairs.
!paragraphIf State is configured to auto-redirect, the window/tab will redirect to either same url or a custom url.
!paragraphReloading the page is by far the more user-friendly, unless the visitor simply meets a 403 (not allowed to use this page).',
    array('!break' => '<br/>', '!paragraph' => '</p><p>', '!emph' => '<em>', '!_emph' => '</em>')
) . '</p>
<h5>' . t('Perpetual redirects/reloads?') . '</h5>
<p>' . t(
    'Possible for form expiration. Not possible for session expiration, because State destroys a session when it expires.',
    array('!break' => '<br/>', '!paragraph' => '</p><p>')
) . '</p>
<h5>' . t('Check interval') . '</h5>
<p>' . t(
    'For every N seconds, check that window/tab session is alive and current, and form hasn\'t expired,
!breakand - if user-activity conditioned session prolongation - whether the visitor has been active since last check.
!breakRecommended value, if session timeout (life on server) is 1800 (half an hour) or more: 300 (~ 5 minutes).
!breakState will also perform AJAX requests at this interval, whenever the prolongation conditions are met.
!paragraphSetting this value correctly is a matter of balancing contradictory interests. Too low: too many requests to the server.
!breakToo high: user-activity conditioned prolongation will tend to prolong sessions excessively (because the last activity may have taken place almost the interval time ago).
!paragraphFrontend ensures that the interval is maximally 20% of the session timeout.
!breakAnd the actual intervals will fluctuate approx. +/-20%, due to fuzzy logic algorithm which ensures that the browser also only "calls in" at the set interval when multiple tabs are open.',
    array('!break' => '<br/>', '!paragraph' => '</p><p>')
) . '</p>'
      ),
  );

  //  Get roles.
  //  Make anonymous user, authenticated user and the administrative role the first three.
  $roles = user_roles();
  $roles_sorted = array();
  try {
    //  Dont trust role ids of anonymous user and authenticated user to be standard; find them and use pseudo ids -2 and -1.
    if (($role_id = array_search('anonymous user', $roles)) === FALSE) {
      drupal_set_message($em = 'role [anonymous user] doesnt exist', 'error');
      throw new Exception($em);
    }
    $roles_sorted['-2'] = t('anonymous user');
    unset($roles[$role_id]);

    if (($role_id = array_search('authenticated user', $roles)) === FALSE) {
      drupal_set_message($em = 'role [authenticated user] doesnt exist', 'error');
      throw new Exception($em);
    }
    $roles_sorted['-1'] = t('authenticated user');
    unset($roles[$role_id]);

    if (!array_key_exists($role_id = variable_get('user_admin_role'), $roles)) {
      drupal_set_message($em = 'administrative role [$role_id]', 'error');
      throw new Exception($em);
    }
    $roles_sorted[ $role_admin = $role_id ] = t('!role (Administrative role)', array('!role' => $roles[$role_id]));
    unset($roles[$role_id]);
  }
  catch (Exception $xc) {
    inspect_trace($xc, array('severity' => WATCHDOG_CRITICAL));
    return system_settings_form(array());
  }
  foreach ($roles as $role_id => $name) {
    $roles_sorted[ $role_id ] = $name;
  }

  //  Default roles for user-activity conditioned prolongation: the administrative role.
  $roles_prlngcnd = variable_get('state__prlngcnd', array($role_admin));
  //  Default roles for unconditional prolongation: anonymous user and authenticated user.
  $roles_prlnguncnd = variable_get('state__prlnguncnd', array('-2', '-1'));

  $form['frontend_session_management'] = array(
      '#type' => 'fieldset',
      '#title' => t('Frontend session management'),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
      'state__frntnd' => array(
          '#type' => 'checkbox',
          '#title' => t('Enable frontend session management'),
          '#default_value' => variable_get('state__frntnd', 0),
          '#attributes' => array(
              'autocomplete' => 'off',
          ),
          '#prefix' => '<p>' . t('Session cookie life:') . ' ' . (!$timeout_cookie ? t('session') : ($timeout_cookie . ' ' . t('seconds')))
              . '<br/>'
              . t('Session timeout (life on server):') . ' ' . $timeout_server . ' ' . t('seconds')
              . '</p>'
              . $timeout_warning,
      ),
      'frontend_session_configure' => array(
          '#type' => 'fieldset',
          '#title' => t('Configure frontend session management'),
          '#collapsible' => TRUE,
          '#collapsed' => TRUE,
          'state__enovrrd' => array(
              '#type' => 'textarea',
              '#default_value' => variable_get('state__enovrrd', ''),
              '#maxlength' => 10000,
              '#attributes' => array(
                  'autocomplete' => 'off',
              ),
          ),
          'state__prlnguncnd' => array(
              '#type' => 'checkboxes',
              '#title' => t('Prolong session unconditionally'),
              '#description' => t(
                  '!emphThis mode is not recommended for administrative roles.!_emph',
                  array('!emph' => '<em>', '!_emph' => '</em>')
              ),
              '#options' => $roles_sorted,
              '#default_value' => $roles_prlnguncnd,
              '#attributes' => array(
                  'autocomplete' => 'off',
              ),
              '#prefix' => '<div class="columns-3"><div>',
              '#suffix' => '</div>',
          ),
          'state__prlngcnd' => array(
              '#type' => 'checkboxes',
              '#title' => t('Prolong session conditioned by recent visitor activity'),
              '#title' => t('Conditional prolongation - require visitor activity'),
              '#description' => t(
                  'Specific role(s) in one mode always beat \'!auth\' in the other mode.!breakOtherwise; if \'!auth\' in neither of the modes,!breakand the user has roles in both modes: conditional prolongation rules.',
                  array('!break' => '<br/>', '!auth' => t('authenticated user'))
               ),
              '#options' => $roles_sorted,
              '#default_value' => $roles_prlngcnd,
              '#attributes' => array(
                  'autocomplete' => 'off',
              ),
              '#prefix' => '<div>',
              '#suffix' => '</div>',
          ),
          'state__chcksss' => array(
              '#type' => 'checkbox',
              '#title' => t(
                  'For visitors that don\'t get session prolongation:!break- check that session is current and form hasn\'t expired',
                  array('!break' => '<br/>')
               ),
              '#default_value' => variable_get('state__chcksss', 0),
              '#attributes' => array(
                  'autocomplete' => 'off',
              ),
              '#prefix' => '<div>',
              '#suffix' => '</div></div>',
          ),
          'state__autrdr' => array(
              '#type' => 'checkbox',
              '#title' => t(
                  'Auto-redirect at session/form timeout!break- after having informed the visitor via dialog box',
                  array('!break' => '<br/>')
              ),
              '#description' => t(
                  'Otherwise; the dialog stays open, and the redirect!breakis performed when the visitor closes the dialog.',
                  array('!break' => '<br/>')
               ),
              '#default_value' => variable_get('state__autrdr', 0),
              '#attributes' => array(
                  'autocomplete' => 'off',
              ),
              '#prefix' => '<div class="columns-3"><div>',
              '#suffix' => '</div>',
          ),
          'state__rdrtrg' => array(
              '#type' => 'textfield',
              '#title' => t('Session/form timeout redirect target'),
              '#description' => t(
                  'Leave empty for reloading the page.!breakInternal url - / (~ start page) or /go-somewhere - or external url - http://some.where.',
                  array('!break' => '<br/>')
               ),
              '#default_value' => variable_get('state__rdrtrg', ''),
              '#attributes' => array(
                  'autocomplete' => 'off',
              ),
              '#prefix' => '<div>',
              '#suffix' => '</div></div>',
          ),
          'state__chckntrvl' => array(
              '#type' => 'textfield',
              '#title' => t('Check interval - see help'),
              '#default_value' => variable_get('state__chckntrvl', 300),
              '#size' => 4,
              '#attributes' => array(
                  'autocomplete' => 'off',
              ),
          ),
          'sub_fields__module_state_frntndsttngs' => array(
              '#type' => 'markup',
              '#markup' =>
              //  Override English
              '<div class="title-local-en-override">' . t('Override English session/form expire warnings') . '</div>'
              . '<div class="description-local-en-override">' . t('When translating to other languages, look for the defaults, not these overrides (if any).') . '</div>'

              . '<div class="form-item form-type-textfield local-en-override">
  <label>' . t('Session/form about to expire') . '</label>
  <div class="locale-default">Default: '
              . str_replace(
                  array('<', '>'),
                  array('&lt;', '&gt;'),
                  'This page is about to expire!newline!newlinePlease save now, if you have made any changes.!newlineOtherwise, please !link_startreload the page!link_end.' // NOT t() - is t()'ed in state.js.
              )
              . '</div>
  <input type="text" class="form-text local-en-override" maxlength="255" size="150" value="" name="local_en__preExpire" autocomplete="off">
  <div class="description">'
              . t('Allowed replacements: !rplcs. First sentence delimited by double !newline will be used as dialog title.', array('!rplcs' => '!newline'))
              . '</div></div>'

              . '<div class="form-item form-type-textfield local-en-override">
  <label>' . t('Session/form expired, reload current page') . '</label>
  <div class="locale-default">Default: '
              . str_replace(
                  array('<', '>'),
                  array('&lt;', '&gt;'),
                  'This page has expired!newline!newlinePlease !link_startreload the page!link_end.' // NOT t() - is t()'ed in state.js.
              )
              . '</div>
  <input type="text" class="form-text local-en-override" maxlength="255" size="150" value="" name="local_en__expired_reload" autocomplete="off">
  <div class="description">'
              . t('Allowed replacements: !rplcs. First sentence delimited by double !newline will be used as dialog title.', array('!rplcs' => '!newline, !url'))
              . '<br/>'
              . t('NB: The visitor will only see this message if the browser prohibits Javascript redirect (Internet Explorer: \'META refresh\').')
              . '</div></div>'

              . '<div class="form-item form-type-textfield local-en-override">
  <label>' . t('Session/form expired, redirect to other page - non-empty \'Session/form timeout redirect target\'') . '</label>
  <div class="locale-default">Default: '
              . str_replace(
                  array('<', '>'),
                  array('&lt;', '&gt;'),
                  'This page has expired!newline!newlinePlease !link_startproceed to the start page!link_end.' // NOT t() - is t()'ed in state.js.
              )
              . '</div>
  <input type="text" class="form-text local-en-override" maxlength="255" size="150" value="" name="local_en__expired_redirect" autocomplete="off">
  <div class="description">'
              . t('Allowed replacements: !rplcs. First sentence delimited by double !newline will be used as dialog title.', array('!rplcs' => '!newline, !url'))
              . '<br/>'
              . t('NB: The visitor will only see this message if the browser prohibits Javascript redirect(Internet Explorer: \'META refresh\').')
              . '</div></div>',
          )
      )
  );
  $form['request_counting'] = array(
      '#type' => 'fieldset',
      '#title' => t('Detect request type - page versus AJAX'),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
      'state__cntreqs' => array(
          '#type' => 'checkbox',
          '#title' => t('Detect request type and count requests'),
          '#default_value' => variable_get('state__cntreqs', 0),
          '#attributes' => array(
              'autocomplete' => 'off',
          ),
      ),
      'state__nonpages' => array(
          '#type' => 'textarea',
          '#title' => t('List of non-page paths - like iframed page views, and AJAX calls failing to send the HTTP_X_REQUESTED_WITH header'),
          '#description' => t(
              'Newline separated. Built-in non-page paths are:!break &nbsp; !built_ins',
              array('!break' => '<br/>', '!built_ins' => join('<br/> &nbsp; ', State::nonPagePaths(TRUE)))
           ),
          '#default_value' => variable_get('state__nonpages', ''),
          '#maxlength' => 1000,
          '#cols' => 100,
          '#rows' => 2,
          '#attributes' => array(
              'autocomplete' => 'off',
          ),
      ),
  );
  $form['#submit'][] = 'state_admin_form_submit';
  return system_settings_form($form);
}

/**
 * @param array $form
 * @param array &$form_state
 * @return void
 */
function _state_admin_form_submit($form, &$form_state) {
  //  Get rid of un-selected roles.
  $roles = $form_state['values']['state__prlngcnd'];
  foreach ($roles as $k => $v) {
    if (!$v) {
      unset($roles[$k]);
    }
  }
  $form_state['values']['state__prlngcnd'] = $roles;
  $roles = $form_state['values']['state__prlnguncnd'];
  foreach ($roles as $k => $v) {
    if (!$v) {
      unset($roles[$k]);
    }
  }
  $form_state['values']['state__prlnguncnd'] = $roles;
  //  Fix format of redirect target.
  if (strlen($v = trim($form_state['values']['state__rdrtrg']))) {
    if (!preg_match('/^(\/|http)/', $v)) { // Ensure starting slash if none and not absolute url.
      $v = '/' . $v;
    }
  }
  $form_state['values']['state__rdrtrg'] = $v;
  //  Fix format of non-page paths.
  if (strlen($v = trim($form_state['values']['state__nonpages']))) {
    $built_ins = State::nonPagePaths(TRUE);
    $paths = array();
    $le = count($a = explode("\n", str_replace("\r", '', $v)));
    for ($i = 0; $i < $le; $i++) {
      if (!strlen($v = trim($a[$i]))) {
        continue;
      }
      if ($v{0} == '/') { // Remove starting slash.
        $v = substr($v, 1);
      }
      if (!strlen($v)) {
        continue;
      }
      if ($v{ strlen($v) - 1 } != '/') { // Secure trailing slash.
        $v .= '/';
      }
      if ($v != '/' && !in_array($v, $built_ins)) {
        $paths[] = $v;
      }
    }
    $form_state['values']['state__nonpages'] = $paths ? join("\n", $paths) : '';
  }
}